@@ -11,6 +11,9 @@
 #if !defined(GLZ_DISABLE_SIMD) && (defined(__x86_64__) || defined(_M_X64))
 #if defined(_MSC_VER)
 #include <intrin.h>
+#pragma warning(push)
+#pragma warning( \
+   disable : 4702) // disable "unreachable code" warnings, which are often invalid due to constexpr branching
 #else
 #include <immintrin.h>
 #endif
@@ -125,7 +128,7 @@ namespace glz
          static constexpr auto num_members = reflect<T>::size;
 
          if constexpr ((num_members > 0) && (glaze_object_t<T> || reflectable<T>)) {
-            invoke_table<N>([&]<size_t I>() {
+            for_each<N>([&]<size_t I>() {
                if (bool(ctx.error)) [[unlikely]] {
                   return;
                }
@@ -157,7 +160,7 @@ namespace glz
             });
          }
          else if constexpr (writable_map_t<T>) {
-            invoke_table<N>([&]<size_t I>() {
+            for_each<N>([&]<size_t I>() {
                if (bool(ctx.error)) [[unlikely]] {
                   return;
                }
@@ -329,7 +332,7 @@ namespace glz
          std::memcpy(&b[ix], "[", 1);
          ++ix;
 
-         invoke_table<N>([&]<size_t I>() {
+         for_each<N>([&]<size_t I>() {
             if (get_member(value, get<I>(reflect<T>::values))) {
                std::memcpy(&b[ix], "\"", 1);
                ++ix;
@@ -358,8 +361,15 @@ namespace glz
    struct to<JSON, T>
    {
       template <auto Opts>
-      static void op(auto&&, is_context auto&&, auto&&...) noexcept
-      {}
+      static void op(auto&&, is_context auto&&, auto&& b, auto&& ix) noexcept
+      {
+         if constexpr (check_write_member_functions(Opts)) {
+            constexpr sv type_name = name_v<T>;
+            dump<'"'>(b, ix);
+            dump(type_name, b, ix);
+            dump<'"'>(b, ix);
+         }
+      }
    };
 
    template <is_reference_wrapper T>
@@ -466,20 +476,6 @@ namespace glz
       }
    };
 
-   inline constexpr std::array<uint16_t, 256> char_escape_table = [] {
-      auto combine = [](const char chars[2]) -> uint16_t { return uint16_t(chars[0]) | (uint16_t(chars[1]) << 8); };
-
-      std::array<uint16_t, 256> t{};
-      t['\b'] = combine(R"(\b)");
-      t['\t'] = combine(R"(\t)");
-      t['\n'] = combine(R"(\n)");
-      t['\f'] = combine(R"(\f)");
-      t['\r'] = combine(R"(\r)");
-      t['\"'] = combine(R"(\")");
-      t['\\'] = combine(R"(\\)");
-      return t;
-   }();
-
    template <class T>
       requires str_t<T> || char_t<T>
    struct to<JSON, T>
@@ -541,7 +537,7 @@ namespace glz
                      return value ? value : "";
                   }
                   else {
-                     return value;
+                     return sv{value};
                   }
                }();
 
@@ -575,7 +571,7 @@ namespace glz
                      return *value.data() ? sv{value.data()} : "";
                   }
                   else {
-                     return value;
+                     return sv{value};
                   }
                }();
                const auto n = str.size();
@@ -604,7 +600,6 @@ namespace glz
                // now we don't have to check writing
 
                if constexpr (Opts.raw) {
-                  const auto n = str.size();
                   if (n) {
                      std::memcpy(&b[ix], str.data(), n);
                      ix += n;
@@ -1154,8 +1149,8 @@ namespace glz
             }
 
             using val_t = detail::iterator_second_type<T>; // the type of value in each [key, value] pair
-
-            if constexpr (not always_skipped<val_t>) {
+            constexpr bool write_member_functions = check_write_member_functions(Opts);
+            if constexpr (!always_skipped<val_t> && (write_member_functions || !is_member_function_pointer<val_t>)) {
                if constexpr (null_t<val_t> && Opts.skip_null_members) {
                   auto write_first_entry = [&](auto&& it) {
                      auto&& [key, entry_val] = *it;
@@ -1355,7 +1350,8 @@ namespace glz
             [&](auto&& val) {
                using V = std::decay_t<decltype(val)>;
 
-               if constexpr (check_write_type_info(Opts) && not tag_v<T>.empty() && glaze_object_t<V>) {
+               if constexpr (check_write_type_info(Opts) && not tag_v<T>.empty() &&
+                             (glaze_object_t<V> || (reflectable<V> && !has_member_with_name<V>(tag_v<T>)))) {
                   constexpr auto N = reflect<V>::size;
 
                   // must first write out type
@@ -1365,26 +1361,54 @@ namespace glz
                      dumpn<Opts.indentation_char>(ctx.indentation_level, b, ix);
                      dump<'"'>(b, ix);
                      dump_maybe_empty(tag_v<T>, b, ix);
-                     dump<"\": \"">(b, ix);
-                     dump_maybe_empty(ids_v<T>[value.index()], b, ix);
-                     if constexpr (N == 0) {
-                        dump<"\"\n">(b, ix);
+
+                     using id_type = std::decay_t<decltype(ids_v<T>[value.index()])>;
+
+                     if constexpr (std::integral<id_type>) {
+                        dump<"\": ">(b, ix);
+                        serialize<JSON>::op<Opts>(ids_v<T>[value.index()], ctx, b, ix);
+                        if constexpr (N == 0) {
+                           dump<"\n">(b, ix);
+                        }
+                        else {
+                           dump<",\n">(b, ix);
+                        }
+                        dumpn<Opts.indentation_char>(ctx.indentation_level, b, ix);
                      }
                      else {
-                        dump<"\",\n">(b, ix);
+                        dump<"\": \"">(b, ix);
+                        dump_maybe_empty(ids_v<T>[value.index()], b, ix);
+                        if constexpr (N == 0) {
+                           dump<"\"\n">(b, ix);
+                        }
+                        else {
+                           dump<"\",\n">(b, ix);
+                        }
+                        dumpn<Opts.indentation_char>(ctx.indentation_level, b, ix);
                      }
-                     dumpn<Opts.indentation_char>(ctx.indentation_level, b, ix);
                   }
                   else {
+                     using id_type = std::decay_t<decltype(ids_v<T>[value.index()])>;
+
                      dump<"{\"">(b, ix);
                      dump_maybe_empty(tag_v<T>, b, ix);
-                     dump<"\":\"">(b, ix);
-                     dump_maybe_empty(ids_v<T>[value.index()], b, ix);
-                     if constexpr (N == 0) {
-                        dump<R"(")">(b, ix);
+
+                     if constexpr (std::integral<id_type>) {
+                        dump<"\":">(b, ix);
+                        serialize<JSON>::op<Opts>(ids_v<T>[value.index()], ctx, b, ix);
+                        if constexpr (N > 0) {
+                           dump<R"(,)">(b, ix);
+                        }
                      }
                      else {
-                        dump<R"(",)">(b, ix);
+                        dump<"\":\"">(b, ix);
+                        dump_maybe_empty(ids_v<T>[value.index()], b, ix);
+                        if constexpr (N == 0) {
+                           dump<R"(")">(b, ix);
+                        }
+                        else {
+                           dump<R"(",)">(b, ix);
+                        }
                      }
                   }
                   to<JSON, V>::template op<opening_and_closing_handled<Opts>()>(val, ctx, b, ix);
@@ -1466,7 +1490,7 @@ namespace glz
                dump_newline_indent<Opts.indentation_char>(ctx.indentation_level, args...);
             }
          }
-         invoke_table<N>([&]<size_t I>() {
+         for_each<N>([&]<size_t I>() {
             if constexpr (glaze_array_t<V>) {
                serialize<JSON>::op<Opts>(get_member(value.value, glz::get<I>(meta_v<T>)), ctx, args...);
             }
@@ -1512,7 +1536,7 @@ namespace glz
             }
          }
          using V = std::decay_t<T>;
-         invoke_table<N>([&]<size_t I>() {
+         for_each<N>([&]<size_t I>() {
             if constexpr (glaze_array_t<V>) {
                serialize<JSON>::op<Opts>(get_member(value, glz::get<I>(meta_v<T>)), ctx, args...);
             }
@@ -1578,7 +1602,7 @@ namespace glz
          static constexpr auto N = glz::tuple_size_v<V> / 2;
 
          bool first = true;
-         invoke_table<N>([&]<size_t I>() {
+         for_each<N>([&]<size_t I>() {
             constexpr auto Opts = opening_and_closing_handled_off<ws_handled_off<Options>()>();
             decltype(auto) item = glz::get<2 * I + 1>(value.value);
             using val_t = std::decay_t<decltype(item)>;
@@ -1588,7 +1612,8 @@ namespace glz
             }
 
             // skip
-            if constexpr (always_skipped<val_t>) {
+            constexpr bool write_member_functions = check_write_member_functions(Opts);
+            if constexpr (always_skipped<val_t> || (!write_member_functions && is_member_function_pointer<val_t>)) {
                return;
             }
             else {
@@ -1656,7 +1681,7 @@ namespace glz
          // When merging it is possible that objects are completed empty
          // and therefore behave like skipped members even when skip_null_members is off
 
-         invoke_table<N>([&]<size_t I>() {
+         for_each<N>([&]<size_t I>() {
             // We don't want to dump a comma when nothing is written
             const auto ix_start = ix;
             using Value = core_t<decltype(get<I>(value.value))>;
@@ -1687,7 +1712,7 @@ namespace glz
    inline constexpr size_t fixed_padding = [] {
       constexpr auto N = reflect<T>::size;
       size_t fixed = 2 + 16; // {} + extra padding
-      for_each_short_circuit<N>([&](auto I) -> bool {
+      for_each_short_circuit<N>([&]<auto I>() -> bool {
          using val_t = field_t<T, I>;
          if constexpr (required_padding<val_t>()) {
             fixed += required_padding<val_t>();
@@ -1782,10 +1807,30 @@ namespace glz
             static constexpr auto padding = round_up_to_nearest_16(maximum_key_size<T> + write_padding_bytes);
             if constexpr (maybe_skipped<Opts, T>) {
                bool first = true;
-               invoke_table<N>([&]<size_t I>() {
+               for_each<N>([&]<size_t I>() {
                   using val_t = field_t<T, I>;
 
-                  if constexpr (always_skipped<val_t>) {
+                  if constexpr (meta_has_skip<T>) {
+                     static constexpr meta_context mctx{.op = operation::serialize};
+                     if constexpr (meta<T>::skip(reflect<T>::keys[I], mctx)) return;
+                  }
+                  if constexpr (meta_has_skip_if<T>) {
+                     static constexpr auto key = glz::get<I>(reflect<T>::keys);
+                     static constexpr meta_context mctx{.op = operation::serialize};
+                     decltype(auto) field_value = [&]() -> decltype(auto) {
+                        if constexpr (reflectable<T>) {
+                           return get<I>(t);
+                        }
+                        else {
+                           return get_member(value, glz::get<I>(reflect<T>::values));
+                        }
+                     }();
+                     if (meta<T>::skip_if(field_value, key, mctx)) return;
+                  }
+
+                  constexpr bool write_member_functions = check_write_member_functions(Opts);
+                  if constexpr (always_skipped<val_t> ||
+                                (!write_member_functions && is_member_function_pointer<val_t>)) {
                      return;
                   }
                   else {
@@ -1828,7 +1873,7 @@ namespace glz
                         first = false;
                      }
                      else {
-                        // Null members may be skipped so we cant just write it out for all but the last member
+                        // Null members may be skipped so we can't just write it out for all but the last member
                         if constexpr (Opts.prettify) {
                            std::memcpy(&b[ix], ",\n", 2);
                            ix += 2;
@@ -1865,7 +1910,7 @@ namespace glz
                   maybe_pad<fixed_max_size>(b, ix);
                }
 
-               invoke_table<N>([&]<size_t I>() {
+               for_each<N>([&]<size_t I>() {
                   if constexpr (not fixed_max_size) {
                      if constexpr (Opts.prettify) {
                         maybe_pad(padding + ctx.indentation_level, b, ix);
@@ -2002,3 +2047,7 @@ namespace glz
       return {buffer_to_file(buffer, file_name)};
    }
 }
+
+#if defined(_MSC_VER)
+#pragma warning(pop)
+#endif
