@@ -336,6 +336,28 @@ struct glz::meta<Thing>
    );
 };
 
+// Custom nullable type for testing nullable_value_t support
+struct custom_nullable_value
+{
+   std::optional<double> val{};
+
+   bool has_value() const { return val.has_value(); }
+
+   double& value() { return *val; }
+
+   const double& value() const { return *val; }
+
+   void emplace() { val.emplace(); }
+
+   void reset() { val.reset(); }
+};
+
+struct nullable_value_test_struct
+{
+   custom_nullable_value x{};
+   int y = 42;
+};
+
 void write_tests()
 {
    using namespace ut;
@@ -459,6 +481,52 @@ void write_tests()
       expect(*uni_dbl == *out_dbl);
    };
 
+   "nullable_value_t"_test = [] {
+      std::string out;
+
+      // Test with value
+      nullable_value_test_struct obj{};
+      obj.x.val = 3.14;
+      expect(not glz::write_beve(obj, out));
+
+      nullable_value_test_struct obj2{};
+      expect(!glz::read_beve(obj2, out));
+      expect(obj2.x.has_value());
+      expect(obj2.x.value() == 3.14);
+      expect(obj2.y == 42);
+
+      // Test with null
+      out.clear();
+      obj.x.val = {};
+      expect(not glz::write_beve(obj, out));
+
+      nullable_value_test_struct obj3{};
+      obj3.x.val = 99.9; // Set a value to ensure it gets reset
+      expect(!glz::read_beve(obj3, out));
+      expect(!obj3.x.has_value());
+      expect(obj3.y == 42);
+
+      // Test standalone nullable_value_t
+      out.clear();
+      custom_nullable_value standalone{};
+      standalone.val = 2.71;
+      expect(not glz::write_beve(standalone, out));
+
+      custom_nullable_value standalone2{};
+      expect(!glz::read_beve(standalone2, out));
+      expect(standalone2.has_value());
+      expect(standalone2.value() == 2.71);
+
+      // Test standalone null
+      out.clear();
+      standalone.val = {};
+      expect(not glz::write_beve(standalone, out));
+
+      standalone2.val = 1.0; // Set a value to ensure it gets reset
+      expect(!glz::read_beve(standalone2, out));
+      expect(!standalone2.has_value());
+   };
+
    "map"_test = [] {
       std::string out;
 
